<?php
/*
	SHChat
	by Scripthosting.net
	http://www.shchat.net

	Autor: Jan-Michael Fuhrmann
	info@scripthosting.net
*/

include_once(dirname(__FILE__)."/class.Channel.php");

class Chatinput extends Channel {

	const CHATINPUT_VERSION = "00901";
	
	/**
	 * Runs a Chat Command in the current channel
	 * @param String $input
	 * @return void
	 */
	public function runChatCommand( $input ){
		
		global $db;
		include_once(dirname(__FILE__)."/class.User.php");
		$user = new User();
		
		// Input-Filter
		$input = htmlentities( $this->clean($input) );
		$strlen = strlen( $input );
		
		// Chat-Command
		if( substr($input,0,1) == "/" ){
			//########### USER-Commands ###########//		
			// Whisper
			if( ( substr($input,0,3)=="/w " || substr($input,0,9)=="/whisper " ) && $user->getUserStatus() == 1 && $user->hasPrivilege("WHISPER"))
			{
				$text = explode(" ",$input);
				$input = substr($input,strlen($text[0])+strlen($text[1])+2);
				$whisperto = $text[1];
		
				if( $user->exists($whisperto) && $user->isOnline($whisperto) )
				{
					$this->addPrivateMsg($input,$whisperto);
					$user->updateCharCount($strlen);
				}		
				else{
					$this->addErrorMsg("Der Benutzer <b>{$whisperto}</b> wurde nicht gefunden!");
				}
			}
		
			// Join
			elseif( ( substr($input,0,3)=="/j " || substr($input,0,6)=="/join " ) && $user->getUserStatus() == 1 && $user->hasPrivilege("JOIN"))
			{
				$text = explode(" ",$input);
				$newchannel = $user->clean( substr($input,strlen($text[0])+1) );
				$isFull = $this->isFull( $newchannel );
				$user_level = $user->getUserLevel();
				$user_name = $user->getUserName();
		
				if( $this->channelExists( $newchannel ) && $user_level >= $this->getMinLevel() )
				{					
					if( $isFull && !$user->isChatAdmin($user_name)){
						$this->addErrorMsg("Der Channel <b>". $newchannel . "</b> ist voll!");
					}			
					else{
						$oldchannel = $this->getChannelName();
					
						if( $newchannel != $oldchannel && $this->isJoinable($this->getChannelId(),$user_level) && !$isFull )
						{
							$user->setLastLogin();
							$this->addSystemMsg("<i><b>{$user_name}</b> verlsst den Channel!</i>");
							$this->setChannelId( $this->getChannelId($newchannel) );	// Aktuellen Channel auf ID setzen
							
							if( !$sep || $sep && $user_level > 2 ){
								$this->addUser( $user_name, $user_level );	// User dem Channel hinzufgen
							}	
							$this->addSystemMsg("<i><b>{$user_name}</b> betritt den Channel!</i>",$newchannel);
						}
					}
				
				}			
				else{
					$this->addErrorMsg("Der Channel <b>". $newchannel . "</b> wurde nicht gefunden!");
				}
			}
		
			// Autoscroll
			elseif(substr($input,0,11)=="/autoscroll" && $user->getUserStatus() == 1)
			{
				if(!$user->getUserAutoScroll()){
					$user->setUserAutoScroll($user->getUserId(), 1);
				}
				else {
					$user->setUserAutoScroll($user->getUserId(), 0);
				}
			}
		
			// SEP
			elseif(substr($input,0,5)=="/sep " && $user->getUserStatus() == 1 && $user->hasPrivilege("SEP"))
			{
				$text = explode(" ",$input);
		
				if(chop($text[1])){
					
					$sep = $user->clean($text[1]);
					$this->addSeparee($sep);
										
					$newchannel = $user->clean($sep);
					$oldchannel = $user->clean($this->getChannelName());
				
					if( $newchannel != $oldchannel && $user->getUserStatus() == 1 && $this->isJoinable($this->getChannelId(),$user->getUserLevel()) && ($this->getOnlineUsers($newchannel) < $this->getUserLimit() || $this->getUserLimit() == 0) )
					{
						$user->setLastLogin();		
						$this->addSystemMsg("<i><b>{$user->getUserName()}</b> verlsst den Channel!</i>");
						$this->setChannelId( $this->getChannelId($newchannel) );	// Aktuellen Channel auf ID setzen
						
						if( $user->getUserLevel() > 2 ){
							$this->addUser( $user->getUserName(), $user->getUserLevel() );	// User dem Channel hinzufgen
						}	
						elseif( $user->getUserLevel() == 1 ){
							$this->addUser( $user->getUserName(), 10 );	// User dem Channel hinzufgen
						}
						$this->addSystemMsg("<i><b>{$user->getUserName()}</b> betritt den Channel!</i>",$newchannel);
					}
				}
			}
		
			// Invite
			elseif( ( substr($input,0,8)=="/invite " OR substr($input,0,5)=="/inv " ) && $user->getUserStatus() == 1 && $user->hasPrivilege("INVITE"))
			{
				$text = explode(" ",$input);
		
				if($text[1] != "" && $text[1] != $user->getUserName() && $user->isOnline($text[1])){
					$invite_text = $user->getUserName() ." hat dich in den Channel ". $this->getChannelName() ." eingeladen! Wenn du ihn betreten m&ouml;chtest klicke <a href=\"javascript:join(\'". $this->getChannelName() ."\');\">hier</a>.";
					$invite_text2 = "Du hast ". $text[1] ." eingeladen den Channel ". $this->getChannelName() ." zu betreten!";
					$this->addChatMsg($invite_text,"WHISPER","Chatbot","#000000", $text[1]);
					$this->addErrorMsg($invite_text2);
					$user->updateCharCount($strlen);
				}
				elseif($text[1] && $text[1] == $user->getUserName()){
					$this->addErrorMsg("Hast du lange Weile oder warum l&auml;dst du dich selber ein?");
				}
				elseif(!$user->isOnline($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
				else{
					$this->addErrorMsg("Und WEN willst du einladen?");
				}
			}
		
			// AWAY
			elseif(substr($input,0,5)=="/away" && $user->getUserStatus() == 1)
			{
				$text = str_replace("/away", "",substr($input,0,230));
		
				if(!$user->isAway() && trim($text) != ""){
					$user->setAway(trim($text));
					$this->addSystemMsg("<i><b>". $user->getUserName() ."</b> ist nun abwesend! Grund: ". $text ."</i>");
				}
				elseif(!$user->isAway() && trim($text) == "" ){
					$user->setAway("Der Benutzer hat keine Nachricht hinterlassen.");
					$this->addSystemMsg("<i><b>". $user->getUserName() ."</b> ist nun abwesend!</i>");
				}
				else {
					$user->setAway("");
					$this->addSystemMsg("<i><b>". $user->getUserName() ."</b> ist wieder da!</i>");
				}
			}
			
			// Color
			elseif(substr($input,0,7)=="/color " ){
				
				$text = explode(" ",$input);
				$color = $this->clean($text[1]);
				
				if( ereg('^#?([a-f]|[A-F]|[0-9]){3}(([a-f]|[A-F]|[0-9]){3})?$',$color) ){
					$user->setUserColor($color);
				}
			}
		
			// Quit
			elseif(substr($input,0,5)=="/quit" )
			{
				$user->logout();
				//header("Location: system/logout.php");
			}
		
			//########### OPERATOR-Commands ###########//		
			// Bot-Request
			elseif(substr($input,0,11)=="/botrequest" && $user->hasPrivilege("REQUEST") )
			{
				if(!$user->isInChannel("Chatbot")){
					$this->addChatbot($this->getChannelName());
					$this->addSystemMsg("<i><b>Chatbot</b> betritt den Channel!</i>");
				}			
				else{
					$this->addSystemMsg("Der Chatbot ist bereits in diesem Channel!");
				}
			}
		
			// Bot-Remove
			elseif(substr($input,0,10)=="/botremove" && $user->hasPrivilege("REQUEST"))
			{
				if($user->isInChannel("Chatbot")){
					$this->removeChatbot($this->getChannelName());
					$this->addSystemMsg("<i><b>Chatbot</b> verlsst den Channel!</i>");
				}			
				else{
					$this->addSystemMsg("Der Chatbot ist nicht in diesem Channel!");
				}
			}
		
			// Chatbot Command
			elseif(substr($input,0,9)=="/chatbot " && $user->hasPrivilege("CHATBOT"))
			{
				if($user->isInChannel("Chatbot")){
					$text = str_replace("/chatbot ", "",$input);
					$this->addSystemMsg($text);
				}
			}
		
			// Voice
			elseif( ( substr($input,0,3)=="/v " || substr($input,0,7)=="/voice " ) && $user->getUserStatus() == 1 && $user->hasPrivilege("VOICE"))
			{
				$text = explode(" ",$input);
		
				if($user->isInChannel($text[1]) && !$user->isChatAdmin($text[1])){
					$user->setOnlineLevel($text[1], 2);
					$this->addSystemMsg("<b>". $text[1] ."</b> wurden von <b>". $user->getUserName() . "</b> Voice-Rechte verliehen!");
				}			
				elseif($user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> hat bereits einen hheren Level!");
				}			
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}
		
			// Unvoice
			elseif( ( substr($input,0,4)=="/uv " || substr($input,0,9)=="/unvoice " ) && $user->getUserStatus() == 1 && $user->hasPrivilege("UNVOICE"))
			{
				$text = explode(" ",$input);
		
				if($user->isInChannel($text[1]) && !$user->isChatAdmin($text[1])){
					$user->setOnlineLevel($text[1], 1);
					$this->addSystemMsg("<b>". $text[1] ."</b> wurden von <b>". $user->getUserName() . "</b> die Voice-Rechte entzogen!");
				}			
				elseif($user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> hat bereits einen hheren Level!");
				}			
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}
		
			// Kick
			elseif(substr($input,0,6)=="/kick " && $user->getUserStatus() == 1 && $user->hasPrivilege("KICK"))
			{
				$text = explode(" ",$input);
		
				if($user->exists($text[1]) && $user->isOnline($text[1]) && !$user->isChatAdmin($text[1]))
				{
					$grund = substr($input,strlen($text[0])+strlen($text[1])+2);
		
					if(chop($text[2])){
						$this->addSystemMsg("<b>". $text[1] ."</b> wurde von <b>{$user->getUserName()}</b> aus dem Channel gekickt! Grund: {$grund}");
					}
					elseif(!chop($text[2]))			{
						$this->addSystemMsg("<b>". $text[1] ."</b> wurde von <b>{$user->getUserName()}</b> aus dem Channel gekickt!");
					}
					$this->removeUser($text[1]);
					$user->logout($text[1]);	// Benutzer als ausgeloggt setzen
				}
				elseif($user->exists($text[1]) && $user->isOnline($text[1]) && $user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> ist Administrator und kann nicht gekickt werden!");
				}
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}
		
			// Ban
			elseif(substr($input,0,5)=="/ban " && $user->getUserStatus() == 1 && $user->hasPrivilege("BAN"))
			{
				$text = explode(" ",$input);
		
				if($user->isOnline($text[1]) && !$user->isChatAdmin($text[1]))
				{
					$grund = substr($input,strlen($text[0])+strlen($text[1])+2);

					if(chop($text[2])){
						$this->addSystemMsg("<b>". $text[1] ."</b> wurde von <b>". $user->getUserName() . "</b> aus dem Channel gekickt und aus dem Chat verbannt! Grund: {$grund}");
					}
					elseif(!chop($text[2]))	{
						$this->addSystemMsg("<b>". $text[1] ."</b> wurde von <b>". $user->getUserName() . "</b> aus dem Channel gekickt und aus dem Chat verbannt!");
					}
					$this->removeUser($text[1]);
					$user->setBanned($text[1]);	// Benutzerstatus auf "verbannt" setzen
					// equals: $user->setUserStatus($text[1], 2);
				}
				elseif($user->isOnline($text[1]) && $user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> ist Administrator und kann nicht verbannt werden!");
				}
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}
		
			// Mod
			elseif(substr($input,0,4)=="/mod" && $user->getUserStatus() == 1 && $user->hasPrivilege("MOD"))
			{
				if($this->isModerated()){
					$this->setModerated($this->getChannelName(), 0);
					$this->addSystemMsg("Dieser Channel ist nun nicht mehr moderiert!");
				}			
				elseif(!$this->isModerated()){
					$this->setModerated($this->getChannelName(), 1);
					$this->addSystemMsg("Dieser Channel ist nun moderiert!");
				}
			}
		
			// Hidden
			elseif(substr($input,0,7)=="/hidden" && $user->getUserStatus() == 1 && $user->hasPrivilege("HIDDEN"))
			{
				if($this->isHidden()){
					$this->setHidden($this->getChannelName(), 0);
					$this->addSystemMsg("Dieser Channel ist nun nicht mehr versteckt!");
				}			
				elseif(!$this->isHidden()){
					$this->setHidden($this->getChannelName(), 1);
					$this->addSystemMsg("Dieser Channel ist nun versteckt!");
				}
			}
		
			// Limit
			elseif(substr($input,0,7)=="/limit " && $user->getUserStatus() == 1 && $user->hasPrivilege("LIMIT"))
			{
				$text = explode(" ",$input);
		
				if($text[1]){
					$this->setUserLimit($this->getChannelName(), $text[1]);
					$this->addSystemMsg("Dieser Channel ist nun auf ". $text[1] ." Benutzer limitiert!");
				}			
				elseif(!$text[1] && $this->getUserLimit() >= 1 && $this->getUserLimit() != 9999 ){
					$this->setUserLimit($this->getChannelName(), 0);
					$this->addSystemMsg("Dieser Channel ist nun nicht mehr limitiert!");
				}
			}
			
			// Silent
			elseif( ( substr($input,0,9)=="/silence " || substr($input,0,3)=="/s " ) && $user->getUserStatus() == 1 && $user->hasPrivilege("SILENCE") )
			{
				$text = explode(" ",$input);
				$name = $this->clean($text[1]);
				(int)$time = $this->clean($text[2]);

				if( $name != "" && $time > 0 && $user->exists($name) && !$user->isChatAdmin($name) ){
					$user->setSilence($name,($time*60)+time());
					$this->addSystemMsg("<b>".$name."</b> wurde von {$user->getUserName()} f&uuml;r {$time} Minuten die Redeerlaubnis entzogen!");
				}
				elseif( $name != "" && $time == 0 && $user->exists($name) ){
					$user->setSilence($name,time());
					$this->addSystemMsg($name . " darf nun wieder sprechen!");
				}
				elseif( $user->isChatAdmin($name) ){
					$this->addErrorMsg("Dieser Benutzer lsst sich das Reden nicht verbieten!");
				}
				else{
					$this->addErrorMsg("Der Benutzer <b>". $name . "</b> wurde nicht gefunden!");
				}
			}
		
			//########### ADMINISTRATOR-Commands ###########//		
			// BROADCAST
			elseif(substr($input,0,4)=="/bc " && $user->getUserStatus() == 1 && $user->hasPrivilege("BROADCAST")){
					$input = str_replace("/bc ", "",$input);
					$this->addBroadcastMsg($input);
			}
						
			// Op
			elseif(substr($input,0,4)=="/op " && $user->getUserStatus() == 1 && $user->hasPrivilege("OP"))
			{
				$text = explode(" ",$input);
		
				if($user->exists($text[1]) && $user->isOnline($text[1]) && !$user->isChatAdmin($text[1])){
					$user->setUserLevel($text[1],20);
					$user->setOnlineLevel($text[1],20);					
		
					$this->addSystemMsg("<b>". $text[1] ."</b> wurden von <b>". $user->getUserName() . "</b> Operator-Rechte verliehen!");
				}			
				elseif($user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> hat bereits einen hheren Level!");
				}			
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}				
		
			// DeOp
			elseif(substr($input,0,6)=="/deop " && $user->getUserStatus() == 1 && $user->hasPrivilege("DEOP"))
			{
				$text = explode(" ",$input);
		
				if($user->exists($text[1]) && $user->isOnline($text[1]) && !$user->isChatAdmin($text[1])){
					$user->setOnlineLevel($text[1], 1);
					$user->setUserLevel($text[1], 1);		
					$this->addSystemMsg("<b>". $text[1] ."</b> wurden von <b>". $user->getUserName() . "</b> die Operator-Rechte entzogen!");
				}			
				elseif($user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> ist Administrator und kann nicht entmachtet werden!");
				}			
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}			
		
			// ChanOp
			elseif(substr($input,0,8)=="/chanop " && $user->getUserStatus() == 1 && $user->hasPrivilege("CHANOP"))
			{
				$text = explode(" ",$input);
		
				if($user->exists($text[1]) && $user->isOnline($text[1]) && !$user->isChatAdmin($text[1])){
					$user->setOnlineLevel($text[1], 10);
					$this->addSystemMsg("<b>". $text[1] ."</b> wurden von <b>". $user->getUserName() . "</b> Channel-Operator-Rechte verliehen!");
				}
				elseif($user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> hat bereits einen hheren Level!");
				}
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}
		
			// DeChanOp
			elseif(substr($input,0,10)=="/dechanop " && $user->getUserStatus() == 1 && $user->hasPrivilege("DECHANOP"))
			{
				$text = explode(" ",$input);
		
				if($user->exists($text[1]) && $user->isOnline($text[1]) && !$user->isChatAdmin($text[1])){
					$user->setOnlineLevel($text[1], 1);	// Benutzerlevel auf 1 setzen
					$this->addSystemMsg("<b>". $text[1] ."</b> wurden von <b>". $user->getUserName() . "</b> die Channel-Operator-Rechte entzogen!");
				}			
				elseif($user->isChatAdmin($text[1])){
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> ist Administrator und kann nicht entmachtet werden!");
				}			
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}
		
			// Unban
			elseif(substr($input,0,7)=="/unban " && $user->getUserStatus() == 1 && $user->hasPrivilege("UNBAN"))
			{
				$text = explode(" ",$input);
		
				if($user->exists($text[1]) && $user->isOnline($text[1])){
					$this->addSystemMsg("Der Chatban von <b>". $text[1] ."</b> wurde von <b>". $user->getUserName() . "</b> aufgehoben!");
					$user->setUserStatus($text[1], 0);	// Benutzer "entbannen"
				}			
				else{
					$this->addErrorMsg("Der Benutzer <b>". $text[1] . "</b> wurde nicht gefunden!");
				}
			}		
			######################			
		}
		// Chat-Message
		elseif( $user->getUserStatus() == 1 && $user->hasPrivilege("WRITE") && !$user->isSilent() )
		{
			if(!$this->isModerated() || $this->isModerated() && $user->hasPrivilege("MODERATE"))
			{			
				if(!$user->isAway()){					
					$this->addChatMsg($input);
					$user->updateCharCount($strlen);
				}
				elseif($user->isAway()){					
					$user->setAway("");	// Away-Status deaktivieren
					$this->addSystemMsg("<i><b>". $user->getUserName() ."</b> ist wieder da!</i>");
					$this->addChatMsg($input);
					$user->updateCharCount($strlen);				
				}
			}
			else {
				$this->addErrorMsg("Sie haben in diesem Channel keine Schreibrechte!");
			}
		}
		elseif( $user->isSilent() ){
			$silence = date("d.m.Y H:i:s",$user->getUserSilence());
			$this->addErrorMsg("Ihnen wurde bis {$silence} Redeverbot erteilt!");
		}		
	}
	
	
	/**
	 * Sends a chat message to the current or given channel
	 * @param String $input
	 * @param String $channel optional
	 * @param String $username optional
	 * @param String $color optional
	 * @param String $whisperto optional
	 * @return void
	 */
	public function addChatMsg( $input, $channel = "", $username = "", $color = "", $whisperto = "" ){
		
		global $db;
		include_once(dirname(__FILE__)."/class.User.php");
		
		$user = new User();
		$channel = ( $channel == "" )	?	$this->getChannelName()	:	$channel;
		$username = ( $username == "" )	?	$user->getUserName()	:	$username;
		$color = ( $color == "" )		?	$user->getUserColor()	:	$color;
		
		if( $whisperto == "self" ){
			$whisperto = $user->getUserName();
		}
		
		// Smileys umwandeln
		//$input = wordwrap( $input,50," ", true);
		$input = $this->convertUrl($input);
		$input = $this->convertSmileys($input);

		// Chatinput
		$this->query("INSERT INTO {$db["postings"]} (username,message,channel,whisperto,color,time) VALUES ('{$username}', '{$input}', '{$channel}', '{$whisperto}', '{$color}', UNIX_TIMESTAMP())");	
	}
	
	
	/**
	 * Sends a System-Message to the current channel
	 * @param String $input
	 * @param String $channel
	 * @return void
	 */
	public function addSystemMsg( $input, $channel = "" ){
		$this->addChatMsg( $input, $channel, "Chatbot", "#000000" );
	}
	
	
	/**
	 * Sends an error message to the current user
	 * @param $input
	 * @param String $whisperto
	 * @return void
	 */
	public function addErrorMsg( $input ){
		$this->addChatMsg( $input, "SYSTEM", "Chatbot", "#800000", "self" );
	}
	
	
	/**
	 * Sends a private message from the current user to another user
	 * @param $input
	 * @param $whisperto
	 * @return void
	 */
	public function addPrivateMsg( $input, $whisperto ){
		$this->addChatMsg($input, "WHISPER", "", "", $whisperto);
	}
	
	
	/**
	 * Sends a Broadcast-Message to the current channel
	 * @param $input
	 * @return void
	 */
	public function addBroadcastMsg( $input ){
		$this->addChatMsg($input, "BROADCAST");
	}
}
?>